@@ -27,17 +27,16 @@ from octobot_evaluators.api.evaluators import (
     get_evaluator_classes_from_type,
     get_evaluators_time_frames,
     create_matrix,
-    create_evaluators,
     stop_evaluator,
     stop_evaluator_channel,
     stop_all_evaluator_channels,
-    create_evaluator,
     initialize_evaluators,
-    create_all_type_evaluators,
+    create_and_start_all_type_evaluators,
 )
 from octobot_evaluators.api.initialization import (
     init_time_frames_from_strategies,
     init_required_candles_count_from_evaluators_and_strategies,
+    get_activated_evaluators,
     get_activated_strategies_classes,
     get_activated_TA_evaluators_classes,
     get_activated_real_time_evaluators_classes,
@@ -63,15 +62,14 @@ __all__ = [
     "get_evaluator_classes_from_type",
     "get_evaluators_time_frames",
     "create_matrix",
-    "create_evaluators",
     "stop_evaluator",
     "stop_evaluator_channel",
     "stop_all_evaluator_channels",
-    "create_evaluator",
     "initialize_evaluators",
-    "create_all_type_evaluators",
+    "create_and_start_all_type_evaluators",
     "init_time_frames_from_strategies",
     "init_required_candles_count_from_evaluators_and_strategies",
+    "get_activated_evaluators",
     "get_activated_strategies_classes",
     "get_activated_TA_evaluators_classes",
     "get_activated_real_time_evaluators_classes",
