@@ -13,7 +13,6 @@
 #
 #  You should have received a copy of the GNU Lesser General Public
 #  License along with this library.
-
 import time
 
 import octobot_tentacles_manager.api as api
@@ -23,11 +22,16 @@ import async_channel.constants as channel_constants
 import async_channel.enums as channel_enums
 
 import octobot_commons.constants as common_constants
+import octobot_commons.databases as databases
+import octobot_commons.errors as commons_errors
 import octobot_commons.tentacles_management as tentacles_management
 
 import octobot_evaluators.evaluators.channel as evaluator_channels
 import octobot_evaluators.constants as constants
 import octobot_evaluators.matrix as matrix
+import octobot_evaluators.evaluators.evaluator_factory as evaluator_factory
+
+import octobot_evaluators.util as util
 
 
 class AbstractEvaluator(tentacles_management.AbstractTentacle):
@@ -86,6 +90,46 @@ class AbstractEvaluator(tentacles_management.AbstractTentacle):
         # Define evaluators default consumer priority level
         self.priority_level: int = channel_enums.ChannelConsumerPriorityLevels.MEDIUM.value
 
+        self.consumers = []
+
+        # True when this evaluator is only triggered on closed candles
+        self.is_triggered_after_candle_close = False
+
+        # Other evaluators that might have been called by this evaluator.
+        # This evaluator is now responsible for managing their cache
+        self.called_nested_evaluators = set()
+
+    def post_init(self, tentacles_setup_config):
+        """
+        Automatically called after __init__ when post_init is True (default) in evaluator_factory
+        Override when necessary
+        :param tentacles_setup_config: the tentacles_setup_config __init__ argument
+        :return: None
+        """
+        pass
+
+    @classmethod
+    async def single_evaluation(
+            cls,
+            tentacles_setup_config: tm_configuration.TentaclesSetupConfiguration,
+            specific_config: dict,
+            ignore_cache=False,
+            should_trigger_post_init=False,
+            **kwargs):
+        evaluator_instance = evaluator_factory.create_temporary_evaluator_with_local_config(
+            cls, tentacles_setup_config, specific_config, should_trigger_post_init)
+        evaluation, error = await evaluator_instance.evaluator_manual_callback(ignore_cache=ignore_cache, **kwargs)
+        return evaluation, error, evaluator_instance
+
+    async def evaluator_manual_callback(self, **kwargs):
+        """
+        Override this method to define the appropriate behavior when this evaluator is being
+        called manually
+        :param kwargs: keyword arguments to be used for evaluation
+        :return: the evaluation value
+        """
+        raise NotImplementedError("evaluator_manual_callback is not implemented")
+
     @staticmethod
     def get_eval_type():
         """
@@ -126,9 +170,26 @@ class AbstractEvaluator(tentacles_management.AbstractTentacle):
     def get_required_candles_count(cls, tentacles_setup_config: tm_configuration.TentaclesSetupConfiguration):
         return api.get_tentacle_config(tentacles_setup_config, cls).get(
             common_constants.CONFIG_TENTACLES_REQUIRED_CANDLES_COUNT,
-            -1
+            common_constants.DEFAULT_IGNORED_VALUE
         )
 
+    def get_trigger_time_frames(self):
+        return self.specific_config.get(common_constants.CONFIG_TRIGGER_TIMEFRAMES, common_constants.CONFIG_WILDCARD)
+
+    @staticmethod
+    def invalidate_cache_on_code_change():
+        # False is not yet supported
+        return True
+
+    @staticmethod
+    def invalidate_cache_on_config_change():
+        # False is not yet supported
+        return True
+
+    @classmethod
+    def use_cache(cls):
+        return False
+
     def _get_tentacle_registration_topic(self, all_symbols_by_crypto_currencies, time_frames, real_time_time_frames):
         currencies = [self.cryptocurrency]
         symbols = [self.symbol]
@@ -141,6 +202,11 @@ class AbstractEvaluator(tentacles_management.AbstractTentacle):
                        for currency_symbol in currency_symbols]
         if self.get_is_time_frame_wildcard():
             available_time_frames = time_frames
+        trigger_timeframes = self.get_trigger_time_frames()
+        if trigger_timeframes != common_constants.CONFIG_WILDCARD:
+            available_time_frames = [tf
+                                     for tf in available_time_frames
+                                     if tf in trigger_timeframes]
         return currencies, symbols, available_time_frames
 
     def initialize(self, all_symbols_by_crypto_currencies, time_frames, real_time_time_frames):
@@ -172,7 +238,9 @@ class AbstractEvaluator(tentacles_management.AbstractTentacle):
                                    eval_note=None,
                                    eval_time=0,
                                    notify=True,
-                                   origin_consumer=None) -> None:
+                                   origin_consumer=None,
+                                   cache_client=None,
+                                   cache_if_available=True) -> None:
         """
         Main async method to notify matrix to update
         :param cryptocurrency: evaluated cryptocurrency
@@ -182,12 +250,22 @@ class AbstractEvaluator(tentacles_management.AbstractTentacle):
         :param eval_time: the time of the evaluation if relevant, default is 0
         :param notify: if true, will trigger matrix consumers
         :param origin_consumer: the sender consumer if it doesn't want to be notified
+        :param cache_client: an existing cache client to avoid creating a local one
+        :param cache_if_available: when True, if the evaluator is using cache, its value will be cached
         :return: None
         """
         try:
             if eval_note is None:
                 eval_note = self.eval_note if self.eval_note is not None else common_constants.START_PENDING_EVAL_NOTE
 
+            if self.use_cache():
+                cache_client = cache_client or util.local_cache_client(self, symbol, time_frame)
+                if self.eval_note == common_constants.DO_NOT_OVERRIDE_CACHE:
+                    self.eval_note, missing = await cache_client.get_cached_value(cache_key=eval_time)
+                    cache_client.ensure_no_missing_cached_value(missing)
+                    eval_note = self.eval_note
+                elif cache_if_available and eval_note != common_constants.DO_NOT_CACHE:
+                    await cache_client.set_cached_value(eval_note, cache_key=eval_time, flush_if_necessary=True)
             self.ensure_eval_note_is_not_expired()
             await evaluator_channels.get_chan(constants.MATRIX_CHANNEL,
                                               self.matrix_id).get_internal_producer().send_eval_note(
@@ -203,6 +281,9 @@ class AbstractEvaluator(tentacles_management.AbstractTentacle):
                 time_frame=time_frame,
                 notify=notify,
                 origin_consumer=origin_consumer)
+        except commons_errors.NoCacheValue:
+            self.logger.warning(f"Evaluation as \"{common_constants.DO_NOT_OVERRIDE_CACHE}\" "
+                                f"but the is no cache to publish an evaluation from")
         except Exception as e:
             # if ConfigManager.is_in_dev_mode(self.config): # TODO
             #     raise e
@@ -232,7 +313,23 @@ class AbstractEvaluator(tentacles_management.AbstractTentacle):
         implement if necessary
         :return: None
         """
-        pass
+        await self.close_caches()
+        for consumer in self.consumers:
+            await consumer.stop()
+
+    async def clear_all_cache(self):
+        for evaluator_identifier in util.get_related_cache_identifiers(self):
+            await databases.CacheManager().clear_cache(evaluator_identifier)
+
+    async def close_caches(self, reset_cache_db_ids=False):
+        for evaluator_identifier in util.get_related_cache_identifiers(self):
+            await databases.CacheManager().close_cache(
+                evaluator_identifier,
+                self.exchange_name,
+                common_constants.UNPROVIDED_CACHE_IDENTIFIER if self.get_is_symbol_wildcard() else self.symbol,
+                common_constants.UNPROVIDED_CACHE_IDENTIFIER if self.get_is_time_frame_wildcard() else self.time_frame.value,
+                reset_cache_db_ids=reset_cache_db_ids
+            )
 
     async def prepare(self) -> None:
         """
@@ -258,6 +355,19 @@ class AbstractEvaluator(tentacles_management.AbstractTentacle):
         """
         self.specific_config = {}
 
+    @classmethod
+    def get_evaluator_priority(cls, tentacles_setup_config) -> float:
+        """
+        Returns the priority of the evaluator (will later be compared to other evaluators).
+        A higher priority evaluator will be called first when multiple evaluators are to
+        be called at the same time.
+        Default priority is DEFAULT_PRIORITY defined in OctoBot-Commons.
+        Order is undefined between evaluators of the same priority.
+        :return: the priority level
+        """
+        return api.get_tentacle_config(tentacles_setup_config, cls).get(common_constants.EVALUATOR_PRIORITY,
+                                                                        common_constants.DEFAULT_EVALUATOR_PRIORITY)
+
     def reset(self) -> None:
         """
         Reset temporary parameters to enable fresh start
