@@ -19,10 +19,16 @@ from octobot_evaluators.util import evaluation_util
 from octobot_evaluators.util.evaluation_util import (
     get_eval_time,
     get_shortest_time_frame,
+    local_trading_context,
+    local_cache_client,
+    get_related_cache_identifiers,
 )
 
 __all__ = [
     "get_eval_time",
     "get_shortest_time_frame",
+    "local_trading_context",
+    "local_cache_client",
+    "get_related_cache_identifiers",
 ]
 
